{ Copyright (c) 1998-2000 Radical Computing Inc.  All rights reserved. }

{ These two components illustrate how to create a remote installer that
  will download new files from the server to the client.  Call the
  DownloadApplication( ApplicationName : String ) methods in the
  TInstallClient component to retrieve files from the server.  On
  the server there must be a text file listing the paths of the
  files to download.  Each file name must be on a separate line.
  The name of the file should be the exact name as the ApplicationName
  parameter passed to the DownloadApplication method.  The file
  should be in the same directory as the server application }

unit Install;

{$R-}

interface

uses
  Windows, SysUtils, Graphics, Classes, Controls, Sckt, Msg, VarProc;

type
  EInstallException = class( Exception );

  TInstallClient = class( TComponent )
  private
    FClient          : TMsgClient;
    FTimeout         : Word;
    FDefaultDir      : String;

    function  GetPort : Integer;
    procedure SetPort( port : Integer );
    function  GetAddress : String;
    procedure SetAddress( address : String );

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure DownloadApplication( ApplicationName : String );

  published
    property Timeout    : Word read FTimeout write FTimeout default 30;
    property DefaultDir : String read FDefaultDir write FDefaultDir;
    property Address    : String read GetAddress write SetAddress;
    property Port       : Integer read GetPort write SetPort;
end;

  TInstallServer = class( TMsgServer )
  private
    function GetFileName( fileName : String ) : String;

  protected
    { Processes a message }
    procedure ProcessMessage( connection : TAsyncMsgConnection ); override;
  end;


implementation


uses
  Forms, Dialogs, InstProg;


function GetExeDirectory : String;
var
  original : String;
  index    : Integer;

begin
  original := Forms.Application.ExeName;
  Result := '';
  index := Pos( '\', original );
  while ( index > 0 ) do begin
    if ( Result <> '' ) then begin
      Result := Result + '\';
    end;
    Result := Result + Copy( original, 1, index - 1 );
    original := Copy( original, index + 1, 500 );
    index := Pos( '\', original );
  end;
end;


function FileTimeStamp( fileName : String ) : TDateTime;
var
  handle : Integer;

begin
  if ( FileExists( fileName ) ) then begin
    handle := FileOpen( fileName, fmOpenRead );
    try
      Result := FileDateToDateTime( FileGetDate( handle ) );
    finally
      FileClose( handle );
    end;
  end else begin
    Result := 0;
  end;
end;


function  TInstallClient.GetPort : Integer;
begin
  Result := FClient.RemotePort;
end;


procedure TInstallClient.SetPort( port : Integer );
begin
  FClient.RemotePort := port;
end;


function TInstallClient.GetAddress : String;
begin
  Result := FClient.RemoteAddress;
end;


procedure TInstallClient.SetAddress( address : String );
begin
  FClient.RemoteAddress := address;
end;


constructor TInstallClient.Create(AOwner: TComponent);
begin
  inherited Create( AOwner );
  FClient := TMsgClient.Create( Self );
  FTimeout := 30;
end;


destructor TInstallClient.Destroy;
begin
  FClient.Free;
  inherited Destroy;
end;


procedure TInstallClient.DownloadApplication( ApplicationName : String );
var
  existing, download : TStrings;
  item               : Integer;
  stream             : TFileStream;
  fileList           : TStrings;

begin
  FClient.Timeout := Timeout;
  FClient.Open;
  try
    with FClient.Connection do begin
      SendMessage( 'CHECK-APPLICATION', 0, [ApplicationName] );
      Receive;
      if ( Msg.MsgType = 'ACK' ) then begin
        existing := TStringList.Create;
        download := TStringList.Create;
        fileList := TStringList.Create;
        try
          if ( FileExists( GetExeDirectory + '\' + ApplicationName ) ) then begin
            existing.LoadFromFile( GetExeDirectory + '\' + ApplicationName );
          end;
          item := 0;
          while ( item < (Msg.ItemCount - 1) ) do begin
            if ( existing.Values[Msg.Items[item]] <> Msg.Items[item + 1] ) then begin
              download.Add( Msg.Items[item]  + '=' + Msg.Items[item + 1] );
            end else begin
              fileList.Add( Msg.Items[item]  + '=' + Msg.Items[item + 1] );
            end;
            Inc( item, 2 );
          end;
          Msg.Clear;
          existing.Clear;
          if ( download.Count > 0 ) then begin
            if ( MessageDlg( 'A new version of this program is available.  Do you want to download it now?', mtConfirmation, [mbYes, mbNo], 0 ) = mrYes ) then begin
              InstallProgress := TInstallProgress.Create( Nil );
              try
                for item := 0 to ( download.Count - 1 ) do begin
                  InstallProgress.lblFile.Caption := download.Names[item];
                  if ( InstallProgress.Visible = False ) then begin
                    InstallProgress.Visible := True;
                  end;
                  InstallProgress.Refresh;
                  SendMessage( 'DOWNLOAD', 0, [download.Names[item]] );
                  Receive;
                  stream := TFileStream.Create( download.Names[item], fmCreate );
                  try
                    VariantToStream( Msg.Items[0], stream );
                  finally
                    stream.Free;
                  end;
                  Clear;
                end;
                fileList.AddStrings( download );
                fileList.SaveToFile( GetExeDirectory + '\' + ApplicationName );
              finally
                InstallProgress.Free;
              end;
            end;
          end;
        finally
          existing.Free;
          download.Free;
          fileList.Free;
        end;
      end;
    end;
  finally
    FClient.Close;
  end;
end;


function TInstallServer.GetFileName( fileName : String ) : String;
begin
  if ( Pos( '\', fileName ) > 0 ) then begin
    Result := fileName;
  end else begin
    Result := GetExeDirectory + '\' + fileName;
  end;
end;


procedure TInstallServer.ProcessMessage( connection : TAsyncMsgConnection );
var
  fileName : String;
  list     : TStrings;
  item     : Integer;
  stream   : TFileStream;
  v        : Variant;
  i        : Integer;

begin
  with Connection.Msg do begin
    try
      if ( MsgType = 'CHECK-APPLICATION' ) then begin
        if ( ItemCount = 1 ) then begin
          fileName := GetFileName( Items[0] );
          Clear;
          if ( not FileExists(fileName) ) then begin
            MsgType := 'ERROR';
            Items[0] := 'Unable to find application configuration file name';
          end else begin
            list := TStringList.Create;
            MsgType := 'ACK';
            try
              list.LoadFromFile( fileName );
              item := 0;
              for i := 0 to ( list.Count - 1 ) do begin
                if ( FileExists(GetExeDirectory + '\' + list[i]) ) then begin
                  Items[item] := list[i];
                  Items[item + 1] := FormatDateTime( 'yyyymmddhhnnss', FileTimeStamp( GetExeDirectory + '\' + list[i] ) );
                  Inc( item, 2 );
                end;
              end;
            finally
              list.Free;
            end;
          end;
        end else begin
          Clear;
          MsgType := 'ERROR';
          Items[0] := 'Invalid parameters';
        end;
        connection.Send;
      end else if ( MsgType = 'DOWNLOAD' ) then begin
        if ( ItemCount = 1 ) then begin
          fileName := GetFileName( Items[0] );
          Clear;
          if ( fileName <> '' ) and ( FileExists(fileName) ) then begin
            stream := TFileStream.Create( fileName, fmOpenRead );
            try
              Stream.Position := 0;
              StreamToVariant( stream, v );
            finally
              stream.Free;
            end;
            Items[0] := v;
            v := NULL;
          end else begin
            MsgType := 'ERROR';
            Items[0] := 'Unable to find requested file name';
          end;
        end else begin
          Clear;
          MsgType := 'ERROR';
          Items[0] := 'Invalid parameters';
        end;
        connection.Send;
      end else begin
        connection.Close;
      end;
    except
      Clear;
      connection.Close;
    end;
    Clear;
  end;
end;


end.

